/**
  Copyright (c) 2007 Freescale Semiconductor
  
  \file     Proximity.c
  \brief    This module performs all the capacitive sensig processes.
  \author   Freescale Semiconductor
  \author   RF, Analog & Sensor Group
  \author   Eduardo Muriel, Ulises Corrales
  \version  1.3
  \date     8/10/2007
  \warning  This module contains a blocking function. In SampleElectrode
  \warning  function, a while statement blocks the CPU for as long as 35us.

  * History: 23/10/2007 - Added average initialization.
                          Changed UpdateElectrode function to UpdateElectrodeStatus.
             22/10/2007 - SampleElectrode function made private for module.
             18/10/2007 - Timer ISR taken to another file
             15/10/2007 - Added timeout condition back with BRCLR instruction.
                          Timeout status handled and reported in UpdateElectrode.
             12/10/2007 - Changed threshold condition for optimization
                          Fixed a bug when discharging electrodes at initialization.
                          Sample values amplified by 4 to improve average resolution.
                          Removed timeout condition from while statement.
             10/10/2007 - Electrode list created to improve flexibility.
              5/10/2007 - File creation.
              
* Copyright (c) 2007, Freescale, Inc.  All rights reserved.
*
*
* No part of this document must be reproduced in any form - including copied,
* transcribed, printed or by any electronic means - without specific written
* permission from Freescale Semiconductor.
*              
                                                    
*/                        

#include "Proximity.h"
#include "Common.h"

UINT8 gau8ElectrodeAvg[N_ELECTRODES];
ELEC_STAT_REG gesElectrodeStatus = 0;
static UINT8 u8Thresholds[N_ELECTRODES] = {30,30,30,30,30,30,30,30};
#ifdef _GUI_
    volatile UINT8 u8Samples[N_ELECTRODES];     /** For the GUI to read the sample values */
#endif    

/** Private functions prototypes */
static UINT8 SampleElectrode(UINT8* pu8ElectrodePort, UINT8 u8ElectrodeMask, UINT8* pu8Sample);

/**
 * \brief   This function initializes the timer used for capacitance measurement and for
 * \brief   periodic sampling. Also discharges all electrodes and initializes average values.
 * \author  Eduardo Muriel
 * \param   None
 * \return  8 bits status code
*/
UINT8 ProximityInit(void)
{
  UINT8 u8Status;
  UINT8 i;
  
  for(i=0;i<N_ELECTRODES;i++)
  {    
    /** Initialize average values with one sample */
    u8Status = SampleElectrode(ELEC_PORT(i),ELEC_PIN(i),&gau8ElectrodeAvg[i]);
    if(u8Status != STAT_OK)
        gau8ElectrodeAvg[i] = 0xFF;     
  }  
    
  return u8Status;
}

/**
 * \brief   This function performs the intergration of the electrode samples and updates.
 * \brief   touch status register. User can choose to update all or just one electrode.
 * \author  Eduardo Muriel, Ulises Corrales
 * \param   u8Electrode - A number representing electrode of interest or a command to update all.
 * \return  8 bits status code
*/
UINT8 UpdateElectrodeStatus(UINT8 u8Electrode)
{
    UINT8 u8Sample;   /** Current sample value */
    UINT8 u8Status;
    UINT8 i;

    if(u8Electrode == ELECTRODE_ALL || u8Electrode < N_ELECTRODES)          /** Check if input parameter is in range*/
    {    
        if(u8Electrode == ELECTRODE_ALL)                                      /** All electrodes will be sampled */
            for(i=0;i<N_ELECTRODES;i++)
            {
                u8Status = SampleElectrode(ELEC_PORT(i),ELEC_PIN(i),&u8Sample);
                
                #ifdef _GUI_
                    u8Samples[i] = u8Sample;
                #endif   
   
                if(u8Status == STAT_OK)             /** If no timeout event occurred while sampling */
                {  
                    if((INT8)(u8Sample - gau8ElectrodeAvg[i]) > (INT8)u8Thresholds[i])
                        gesElectrodeStatus |= 0x01<<i;                                     /** Update pressed status register */
                    else
                        gesElectrodeStatus &= ~(0x01<<i);
           
                    /** Integrate current sample to average */
                    /** Average = Average + (Sample - Average)/8 */
                    gau8ElectrodeAvg[i] = gau8ElectrodeAvg[i] + (((INT8)(u8Sample - gau8ElectrodeAvg[i]))>>2);
                }
                else
                    gesElectrodeStatus &= ~(0x01<<i);
            }
        else                                                                  /** Only one electrode will be sampled */
        { 
            u8Status = SampleElectrode(ELEC_PORT(u8Electrode),ELEC_PIN(u8Electrode), &u8Sample);
  
            #ifdef _GUI_
                    u8Samples[i] = u8Sample;
            #endif   
                
            if(u8Status == STAT_OK)               /** If no timeout event occurred while sampling */                                                 
            {
                if((INT8)(u8Sample - gau8ElectrodeAvg[u8Electrode]) > (INT8)u8Thresholds[u8Electrode])
                    gesElectrodeStatus |= 0x01<<u8Electrode;                             /** Update press status register */
                else
                    gesElectrodeStatus &= ~(0x01<<u8Electrode);
            
                /** Integrate current sample to average */
                /** Average = Average + (Sample - Average)/8 */
                gau8ElectrodeAvg[u8Electrode] = gau8ElectrodeAvg[u8Electrode] + (((INT8)(u8Sample - gau8ElectrodeAvg[u8Electrode]))>>2);
            }
            else
                gesElectrodeStatus &= ~(0x01<<u8Electrode);
        }

        return u8Status;            /** Return status code */    
    }
    else       
        return STAT_OUT_OF_RANGE;   /** Return input parameter out of range error code */
}

/**
 * \brief   This function performs the capacitance measurement on the specified electrode
 * \author  Eduardo Muriel, Ulises Corrales
 * \param   UINT8* pu8ElectrodePort A pointer to a valid MCU port must be provided.
 * \param   UINT8  u8ElectrodeMask  The mask of the bit where electrode is located.
 * \param   UINT8* pu8Sample        Pointer to the variable where counter will be saved.
 * \return  8 bits status code
 * \warning The while statement in this function can block CPU for as long as 35us.
*/
UINT8 SampleElectrode(UINT8* pu8ElectrodePort, UINT8 u8ElectrodeMask, UINT8* pu8Sample)
{   
    ELECTRODE_DISCHARGE(pu8ElectrodePort, u8ElectrodeMask); /** Set initial condition to 0 */
    COUNTER_STOP();                              /** Stop counter in case it was running   */
    COUNTER_RESET();                             /** Reset counter for new measurement     */                                  
    COUNTER_START();                             /** Start counter                         */
    
    /** Set pin as input to begin capacitor charge process */
    ELECTRODE_READ(pu8ElectrodePort, u8ElectrodeMask);  
    
    /** Wait for electrode to reach ~0.7Vdd or for counter to tiemout */
    while(!ELECTRODE_TEST(pu8ElectrodePort, u8ElectrodeMask) && !fTimeout)
        ;
    
    COUNTER_STOP();                              /** Stop counter       */
    *pu8Sample = COUNTER_VALUE;                  /** Save counter value */
    
    ELECTRODE_DISCHARGE(pu8ElectrodePort, u8ElectrodeMask); /** Set initial condition to 0 */
    
    /** Report if measurement was valid or if timeout event ocurred */
    if(fTimeout)
    {
        fTimeout = 0;
        return STAT_TIMEOUT;
    }
    else
        return STAT_OK;
}